unit uNSClient;
// Purpose : Implementation of the service.
// Created : YR - November 2000

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, SvcMgr, Dialogs,
		libSystem, pdh, pdhmsg, ExtCtrls, ScktComp, Registry, JclSYSInfo;

	// Main class
	type TNSClient = class(TService)
      ServerSocket: TServerSocket;
		CPUTimer: TTimer;
    	procedure ServerSocketClientRead(Sender: TObject; Socket: TCustomWinSocket);
      Procedure SetArgvFromRequest(sRequest : string);
     	procedure ServiceStart(Sender: TService; var Started: Boolean);
    	procedure CPUTimerTimer(Sender: TObject);
		procedure ServiceAfterUninstall(Sender: TService);
    	procedure ServiceAfterInstall(Sender: TService);
		private
			sPassword : string;
			query : HQUERY;
	   	cpu_counter : HCOUNTER;
         uptime_counter : HCOUNTER;
         mem_commitLimit_counter : HCOUNTER;
         mem_commitByte_counter : HCOUNTER;
	   	CounterValue: TPdhFmtCounterValue;
         CPUBuffer : TCircularBuffer;
         lnUptime : Int64;
         lnMemCommitLimit : Int64;
         lnMemCommitByte: Int64;
         asArgv : array of string;
     		Procedure CollectData;

         Function FindAnswer(sRequest: string) : String;
		   Function ProcessServReq: string;
		   Function ProcessProcReq: string;
 		public
    		function GetServiceController: TServiceController; override;
		published

    	{ Public declarations }
	end;

  	//Record to hold the information from one thread
  	TThreadInfo = record
		Active       : Boolean;
     	ThreadHandle : integer;
     	ThreadId     : Cardinal;
     	CurrentFile  : string;
  	end;

	const PRG_VERSION = 'NSClient V1.03 - 05.11.2000';
   // Requests constants declaration
	const REQ_VERSION = 1;
	const REQ_CPUTOTAL = 2;
	const REQ_UPTIME = 3;
   const REQ_FREESPACE = 4;
   const REQ_SERVICESTATE = 5;
   const REQ_RUNNINGPROCESS = 6;
	const REQ_MEMORYUSE = 7;
var
  NSClient: TNSClient;

implementation

{$R *.DFM}

	procedure PdhCheck(const Error: Longint);
	// Purpose	: Check wrapper for all PDH queries
	begin
	  if Error <> ERROR_SUCCESS then raise Exception.Create('Error: ' + IntToHex(8, Error));
	end;

	procedure ServiceController(CtrlCode: DWord); stdcall;
	begin
	  NSClient.Controller(CtrlCode);
	end;

	function TNSClient.GetServiceController: TServiceController;
	begin
	  Result := ServiceController;
	end;

	procedure TNSClient.ServiceStart(Sender: TService; var Started: Boolean);
	// Purpose	: Initialize the counters
	// Created	: YR - 21.11.2000
	var
	  Reg: TRegistry;
	begin
		CPUBuffer := TCircularBuffer.Create(17280);

		// Read config informations in the registry
  		Reg := TRegistry.Create;
		Reg.RootKey:= HKEY_LOCAL_MACHINE;
  		if not Reg.KeyExists('\Software\NSClient\Params') then
      begin
	      ServerSocket.port := 1248;
         sPassword := 'None';
      end
   	else
      begin
			Reg.OpenKey('\Software\NSClient\Params',true);
         ServerSocket.port := Reg.ReadInteger('port');
      	sPassword := Reg.ReadString('password');
      end;
    	Reg.CloseKey;
    	Reg.Free;

		// Start listen to the port.
  		ServerSocket.Active:=True;

		// Create the PDH handle to collect CPU and uptime informations
	 	PdhCheck(pdhOpenQuery(nil,0,query));

  		// Initialize the CPU Counter query
		if GetWindowsVersion in [wvWinNT4] then	// Windows NT 4 only
		   PdhCheck(pdhAddCounter(query,PChar('\\' + Get_ComputerName +
		   	'\System\% Total Processor Time'),0,cpu_counter))
      else
	   	PdhCheck(pdhAddCounter(query,PChar('\\' + Get_ComputerName +
	   		'\Processor(_total)\% Processor Time'),0,cpu_counter));

  		// Initialize the uptime counter query
	   PdhCheck(pdhAddCounter(query,PChar('\\' + Get_ComputerName +
	   	'\System\System Up Time'),0,uptime_counter));
      // Initialize the memory counters
		PdhCheck(pdhAddCounter(query,PChar('\\' + Get_ComputerName +
	   	'\Memory\Commit Limit'),0,mem_commitLimit_counter));
		PdhCheck(pdhAddCounter(query,PChar('\\' + Get_ComputerName +
	   	'\Memory\Committed Bytes'),0,mem_commitByte_counter));

	end;

	Procedure TNSClient.CollectData;
	// Purpose	: Get CPU load and add it to a circular buffer
	// Created	: YR - 21.11.2000
	begin
		// Get the informations
		PdhCheck(PdhCollectQueryData(query));

      // Process CPU counter
		PdhCheck(PdhGetFormattedCounterValue(cpu_counter, PDH_FMT_LARGE, nil, CounterValue));
		case CounterValue.CStatus of
			PDH_CSTATUS_NEW_DATA: CPUBuffer.Push(CounterValue.LargeValue);
		   PDH_CSTATUS_VALID_DATA: CPUBuffer.Push(CounterValue.LargeValue);
	   end;

      // Process Uptime counter
		PdhCheck(PdhGetFormattedCounterValue(uptime_counter, PDH_FMT_LARGE, nil, CounterValue));
		case CounterValue.CStatus of
			PDH_CSTATUS_NEW_DATA: lnUptime:=CounterValue.LargeValue;
		   PDH_CSTATUS_VALID_DATA: lnUptime:=CounterValue.LargeValue;
	   end;

      // Process memory counters
		PdhCheck(PdhGetFormattedCounterValue(mem_commitLimit_counter, PDH_FMT_LARGE, nil, CounterValue));
		case CounterValue.CStatus of
			PDH_CSTATUS_NEW_DATA, PDH_CSTATUS_VALID_DATA: lnMemCommitLimit:=CounterValue.LargeValue;
	   end;
		PdhCheck(PdhGetFormattedCounterValue(mem_commitByte_counter, PDH_FMT_LARGE, nil, CounterValue));
		case CounterValue.CStatus of
			PDH_CSTATUS_NEW_DATA, PDH_CSTATUS_VALID_DATA: lnMemCommitByte :=CounterValue.LargeValue;
	   end;


	end;

   Function TNSClient.FindAnswer(sRequest: string) : String;
	// Purpose	: Process all requests from the plugin
	// Created	: YR - 22.11.2000
	begin
      SetArgvFromRequest(sRequest);	// split the string into a string array
      try
			if sPassword = asArgv[0] then
				begin
					// If the first argument is not a number, Delphi will generate an exception
			      case StrToIntDef(asArgv[1],0) of
				      REQ_VERSION: FindAnswer:= PRG_VERSION;
		         	REQ_CPUTOTAL: FindAnswer:= IntToStr(CPUBuffer.GetAverage(StrToIntDef(asArgv[2],60)*12));
  				      REQ_UPTIME: FindAnswer:= IntToStr(lnUptime);
						REQ_FREESPACE: FindAnswer:=IntToStr(GetFreeSpace(strupper(PChar(asArgv[2])))) + '&' +
                  	IntToStr(GetDiskSize(strupper(PChar(asArgv[2]))));
  				      REQ_SERVICESTATE: FindAnswer:= ProcessServReq;
                  REQ_RUNNINGPROCESS: FindAnswer := ProcessProcReq;
                  REQ_MEMORYUSE: FindAnswer := IntToStr(lnMemCommitLimit) + '&' + IntToStr(lnMemCommitByte);
		         else
		     			FindAnswer:= 'ERROR:Unknown request number';
			      end;
            end
         else
				FindAnswer:= 'ERROR:Wrong password';
      except
      	on E: Exception do
         	begin
         	FindAnswer := 'ERROR:Malformed request or internal error. Check EventLog:' +sRequest;
            WriteToEventLog(E.Message);
            end;
      end;
   end;

   Procedure TNSClient.SetArgvFromRequest(sRequest : string);
	// Purpose	: Set the asArgv array according to the request we received
	// Created	: YR - 22.11.2000
	var
      nPos : integer;
      nArgs : integer;
      i: integer;
   begin
		SetLength(asArgv,10);
		for i:=0 to length(asArgv)-1 do asArgv[i]:='';	// Fill the params array with ''

		// Fill the array
      nArgs:=0;
		nPos := Pos('&',sRequest);
		while nPos>0 do begin
      	asArgv[nArgs] := copy(sRequest,1,nPos-1);		// Get the parameter
         sRequest := copy(sRequest, nPos+1, maxint);	// Remove the parameter from the request
         inc(nArgs);             							// inc number of parameters
         if nArgs>=length(asArgv) then    				// increase array if too small
         	setlength(asArgv,length(asArgv)+10);
     		nPos := Pos('&',sRequest);							// look for another parameters
      end;
		asArgv[nArgs]:=sRequest;
		SetLength(asArgv,nArgs+1);								// set the length of the array
   end;

	procedure TNSClient.ServerSocketClientRead(Sender: TObject; Socket: TCustomWinSocket);
	// Purpose	: Receive the TCP requests and send the answer to the plugin
	// Created	: YR - 22.11.2000
	begin
		Socket.SendText(FindAnswer(Socket.ReceiveText));
	end;

	procedure TNSClient.CPUTimerTimer(Sender: TObject);
   // Purpose	: Collect CPU load data
	// Created	: YR - 22.11.2000
	begin
		CollectData;
	end;

   function TNSClient.ProcessServReq: string;
   // Purpose	: Check the state of all services and return a status and
   // 				the faulty services
	// Created	: YR - 27.11.2000
	var
   	i : integer;			// loop's index
      s : string;				// answer
      nState : integer;		// Overall service state
      bShowAll : boolean;	// true is we have to display started services too
   begin
		nState := 0;							// OK State
      s := '';
      bShowAll := False;
		if length(asArgv) >= 4 then	// minimum number of parameters
      begin
			// Do we have to return started services too ?
			if strupper(PChar(asArgv[2])) = 'SHOWALL' then
         	bShowAll:=true;

			// Loop on every service we have to check
	      for i:=3 to length(asArgv)-1 do
				if ServiceRunning('',asArgv[i]) then begin
            	// Service is running
					if bShowAll=True then s := s + ' ' + asArgv[i] + ': Started -'
           	end
            else if (ServiceStopped('',asArgv[i])) then	begin
					// Service is stopped
					s := s + ' ' + asArgv[i] +': Stopped -';
              	nState := 2;
           	end
            else begin
					// service is unknown
            	s := s + ' ' + asArgv[i] +': Unknown -';
               if (nState<1) then nState := 1;
            end;
	      ;

			if length(s)=0 then
      		s := 'All services are running'		// if everything's fine
         else
				SetLength(s,length(s)-1);				// remove trailing -
		end
		else
      begin
      	// not enough parameters. Should never occur, but...
      	s := 'No service to check';
         nState := -1;
      end;
  	   ProcessServReq := IntToStr(nState) + '&' + s;
   end;

   function TNSClient.ProcessProcReq: string;
   // Purpose	: Check if all process are running and
   //					return a status and the faulty processes
	// Created	: YR - 28.11.2000
	var
   	i : integer;			// loop's index
      s : string;				// answer
      nState : integer;		// Overall Process state
      bShowAll : boolean;	// true is we have to display all processes
   begin
		nState := 0;							// OK State
      s := '';
      bShowAll := False;
		if length(asArgv) >= 4 then // minimum number of parameters
      begin
			// Do we have to return running process too ?
			if strupper(PChar(asArgv[2])) = 'SHOWALL' then
         	bShowAll:=true;

			// Loop on every process we have to check
	      for i:=3 to length(asArgv)-1 do
				if ProcessRunning(asArgv[i]) then
            begin
            	// Process is running
					if bShowAll=True then s := s + ' ' + asArgv[i] + ': Running -'
           	end
            else
            begin
					// Process is not running
            	s := s + ' ' + asArgv[i] +': not running -';
               nState := 2;
            end;
	      ;
			if length(s)=0 then
      		s := 'All processes are running'		// if everything's fine
         else
				SetLength(s,length(s)-1);				// remove trailing -
		end
		else begin
      	// not enough parameters
      	s := 'No process to check';
         nState := -1;
      end;
  	   ProcessProcReq := IntToStr(nState) + '&' + s;
   end;

	procedure TNSClient.ServiceAfterUninstall(Sender: TService);
   // Purpose	: Remove the registry entries after the servie is uninstalled
	// Created	: YR - 28.11.2000
	var
		Reg: TRegistry;
	begin
  		Reg := TRegistry.Create;
		Reg.RootKey:= HKEY_LOCAL_MACHINE;

		if Reg.KeyExists('\Software\NSClient\Params') then
	   begin
			Reg.DeleteValue('port');
  			Reg.DeleteValue('password');
			Reg.DeleteKey('\Software\NSClient\Params');
			Reg.DeleteKey('\Software\NSClient');
	   end;
  		WriteToEventLog('NSClient is uninstalled');
	end;

	procedure TNSClient.ServiceAfterInstall(Sender: TService);
   // Purpose	: Add the registry entries after the service is installed
	// Created	: YR - 28.11.2000
	var
		Reg: TRegistry;
	begin
  		Reg := TRegistry.Create;
		Reg.RootKey:= HKEY_LOCAL_MACHINE;

		if not Reg.KeyExists('\Software\NSClient\Params') then
	   begin
			// if the key doesn't exist, let's create it.
			Reg.CreateKey('\Software\NSClient\Params');
			Reg.OpenKey('\Software\NSClient\Params',true);
	      Reg.WriteInteger ('port',1248);
			Reg.WriteString ('password','None');
	   end;
  		WriteToEventLog('NSClient installed');
	end;

end.
